// sintassi: nome_file nome_nuova_sezione dimensioni_sezione

#include <windows.h>
#include <stdio.h>

DWORD CalcAlignment(DWORD Alignment, DWORD TrueSize); 

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	WORD nSection;
	DWORD FileSize, BRW, NameSize, Size;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;
	IMAGE_SECTION_HEADER *ImageSectionHeader;


	// controlla numero argomenti
	if (argc < 4) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
    }
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ | GENERIC_WRITE, 
		FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BRW, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

    // prende le dimensioni
    Size = atoi(argv[3]);

    printf("Creating New Section...\n");

	nSection = ImageNtHeaders->FileHeader.NumberOfSections;

    // aggiorna il numero di sezioni nel FILE HEADER
    ImageNtHeaders->FileHeader.NumberOfSections++;

	ImageSectionHeader = IMAGE_FIRST_SECTION(ImageNtHeaders);

	// aggiorna Size Of Image
    ImageNtHeaders->OptionalHeader.SizeOfImage += 
		CalcAlignment(ImageNtHeaders->OptionalHeader.SectionAlignment, Size);

	// teoricamente dovremmo controllare se c'
	// abbastanza spazio tra la fine della section
	// table e la prima sezione e se non c'
	// dobbiamo ingrandire lo spazio che intercorre
	// tra la header zone e la prima sezione per 
	// inserire la nostra sezione, ma dato che 
	// questo  un esempio lasciamo perdere
	// e prendiamo per buono che lo spazio c'
     
	ZeroMemory(&ImageSectionHeader[nSection],
		IMAGE_SIZEOF_SECTION_HEADER);


	if (strlen(argv[2]) <= IMAGE_SIZEOF_SHORT_NAME)
		NameSize = strlen(argv[2]);
	else
		NameSize = IMAGE_SIZEOF_SHORT_NAME;
 
      // copia il nome della nuova sezione
      memcpy(&ImageSectionHeader[nSection].Name, argv[2], NameSize);


	  // calcola il Virtual Address della nuova sezione
	  ImageSectionHeader[nSection].VirtualAddress =
		  CalcAlignment(ImageNtHeaders->OptionalHeader.SectionAlignment, 
		  (ImageSectionHeader[nSection - 1].VirtualAddress +
		  ImageSectionHeader[nSection - 1].Misc.VirtualSize));
 

	  // Virtual Size = Size vero della sezione
	  // potremmo anche allinearlo per
      ImageSectionHeader[nSection].Misc.VirtualSize = Size;

	  // controllo se la sezione precedente  
	  // allineata correttamente
	  if (ImageSectionHeader[nSection - 1].SizeOfRawData %
		  ImageNtHeaders->OptionalHeader.FileAlignment)
	  {
		  // se no ci penso io
		  ImageSectionHeader[nSection - 1].SizeOfRawData =
			  CalcAlignment(ImageNtHeaders->OptionalHeader.FileAlignment,
			  ImageSectionHeader[nSection - 1].SizeOfRawData);

		  SetFilePointer(hFile, 
			  (ImageSectionHeader[nSection - 1].PointerToRawData +
			  ImageSectionHeader[nSection - 1].SizeOfRawData), NULL,
			  FILE_BEGIN);

		  SetEndOfFile(hFile);
	  }

	  // setto l'address fisico
      ImageSectionHeader[nSection].PointerToRawData = 
		  GetFileSize(hFile, NULL);

	  // e la grandezza fisica
	  ImageSectionHeader[nSection].SizeOfRawData = 
		  CalcAlignment(ImageNtHeaders->OptionalHeader.FileAlignment, Size);

      // caratteristiche (readable)
      ImageSectionHeader[nSection].Characteristics = IMAGE_SCN_MEM_READ;


	  // aggiungo i byte in fondo al file
	  SetFilePointer(hFile, ImageSectionHeader[nSection].SizeOfRawData,
		  NULL, FILE_END);

	  SetEndOfFile(hFile);

	  // salvo le modifiche
	  SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
	  WriteFile(hFile, BaseAddress, FileSize, &BRW, NULL);

	  free(BaseAddress);
	  CloseHandle(hFile);

      printf("\nNew Section Added\n");
 
      return 0;
}

// calcola l'allineamento in base alla grandezza originale

DWORD CalcAlignment(DWORD Alignment, DWORD TrueSize) 
{
	DWORD CalculatedAlignment;

	for(CalculatedAlignment = Alignment; ; CalculatedAlignment 
		+= Alignment) 
	{
		if (TrueSize <= CalculatedAlignment) break;
	}

	return CalculatedAlignment;
}

