// sintassi: nome_file

#include <windows.h>
#include <stdio.h>
#include <delayimp.h>

DWORD VaToOffset(IMAGE_NT_HEADERS *NT, DWORD Va);
DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva);

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	DWORD FileSize, BR, Delay_Offset;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;

	ImgDelayDescr *ImgDelay;

	DWORD Size = 0;
	char *ModName;

	// controlla numero argomenti
	if (argc < 2) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
	}
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ, FILE_SHARE_READ, 
		0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BR, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	if (!ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT].VirtualAddress)
	{
		printf("There is Delay Import Info\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	} 

	// converte in offset fisico l'RVA
	Delay_Offset = RvaToOffset(ImageNtHeaders, 
		ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT].VirtualAddress);

	//  valido?
	if (Delay_Offset == NULL)
	{
		printf("There is Delay Import Info\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	printf("\nDelay Import Info:");


	while (TRUE)
	{
		// prendo l'attuale Delay Descriptor
		ImgDelay = (ImgDelayDescr *) (DWORD)
			(Delay_Offset + Size + (DWORD) BaseAddress);

		// siamo giunti alla fine dell'array?
		if (ImgDelay->pIAT == NULL)
			break;

		// prendo il nome del modulo
		// controllando se ho VA o RVA
		if (ImgDelay->grAttrs & 1)
		{
			ModName = (char *)(DWORD)
				(RvaToOffset(ImageNtHeaders, (DWORD) ImgDelay->szName) +
				(DWORD) BaseAddress);
		}
		else
		{
			ModName = (char *)(DWORD)
				(VaToOffset(ImageNtHeaders, (DWORD) ImgDelay->szName) +
				(DWORD) BaseAddress);
		}

		// stampo solo le info pi importanti
		printf("\n\nName: %s\n"
			"IAT: %X\nINT: %X", ModName,
			ImgDelay->pIAT, ImgDelay->pINT);
		
		Size += sizeof (ImgDelayDescr);
	
	}

	free(BaseAddress);
	CloseHandle(hFile);

	return 0;
}

// per passare da VA a Offset

DWORD VaToOffset(IMAGE_NT_HEADERS *NT, DWORD Va)
{
	return RvaToOffset(NT, 
		(Va - NT->OptionalHeader.ImageBase));
}

// sappiamo a cosa serve

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva)
{
	DWORD Offset = Rva, Limit;
	IMAGE_SECTION_HEADER *Img;
	WORD i;

	Img = IMAGE_FIRST_SECTION(NT);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (i = 0; i < NT->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Rva >= Img[i].VirtualAddress &&
		    Rva < (Img[i].VirtualAddress + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].VirtualAddress;
				Offset += Img[i].PointerToRawData;
			}
				
			return Offset;
		}
	}

	return NULL;
}