// sintassi: nome_file

#include <windows.h>
#include <stdio.h>

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva);

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	DWORD FileSize, BR, IT_Offset;
	UINT x = 0, y;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;
	IMAGE_IMPORT_DESCRIPTOR *ImageImportDescr;

	DWORD *Thunks;
	char *Name;
	IMAGE_IMPORT_BY_NAME *ImgName;


	// controlla numero argomenti
	if (argc < 2) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
    }
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ, FILE_SHARE_READ, 
		0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BR, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	if (!ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress)
	{
		printf("This PE Doesn't contain an IT\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	} 

	// converte in offset fisico l'RVA
	IT_Offset = RvaToOffset(ImageNtHeaders, 
		ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress);

	//  valido?
	if (IT_Offset == NULL)
	{
		printf("This PE Doesn't contain an IT\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	printf("Import Table:\n");

	// ricava la Import Dir
	// e quindi i descriptors
	ImageImportDescr = (IMAGE_IMPORT_DESCRIPTOR *) (IT_Offset +
		(DWORD) BaseAddress);

	// passa in rassegna tutti i descriptors
	while (ImageImportDescr[x].FirstThunk != 0)
	{
		Name = (char *) (RvaToOffset(ImageNtHeaders,
			ImageImportDescr[x].Name) + (DWORD) BaseAddress);

		printf("\nModule Name: %s\n\nFunctions:\n\n", Name);

		// guarda quale array considerare
		Thunks = (DWORD *) (RvaToOffset(ImageNtHeaders,
			ImageImportDescr[x].OriginalFirstThunk != 0 ?
			ImageImportDescr[x].OriginalFirstThunk : 
		    ImageImportDescr[x].FirstThunk) + (DWORD) BaseAddress);

		y = 0;

		// passa in rassegna le funzioni
		while (Thunks[y] != 0)
		{
			//  importata per ordinal?
			if (Thunks[y] & IMAGE_ORDINAL_FLAG)
			{
				printf("Ordinal: %08X\n", (Thunks[y] - 
					IMAGE_ORDINAL_FLAG));
				y++;
				continue;
			}

			ImgName = (IMAGE_IMPORT_BY_NAME *) (RvaToOffset(
				ImageNtHeaders, Thunks[y]) + (DWORD) BaseAddress);

			printf("Name: %s\n", &ImgName->Name);

			y++;
		}

		x++;
	}

	free(BaseAddress);
	CloseHandle(hFile);

	return 0;
}

// sappiamo a cosa serve

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva)
{
	DWORD Offset = Rva, Limit;
	IMAGE_SECTION_HEADER *Img;
	WORD i;

	Img = IMAGE_FIRST_SECTION(NT);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (i = 0; i < NT->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Rva >= Img[i].VirtualAddress &&
		    Rva < (Img[i].VirtualAddress + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].VirtualAddress;
				Offset += Img[i].PointerToRawData;
			}
				
			return Offset;
		}
	}

	return NULL;
}