// sintassi: nome_file

#include <windows.h>
#include <stdio.h>

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva);

int main(int argc, char *argv[]) 
{
	HANDLE hFile;
	BYTE *BaseAddress;
	DWORD FileSize, BR, Reloc_Offset, Reloc_Size;

	IMAGE_DOS_HEADER *ImageDosHeader;
	IMAGE_NT_HEADERS *ImageNtHeaders;

	IMAGE_BASE_RELOCATION *ImageRelocation;

	DWORD Size = 0;

	// controlla numero argomenti
	if (argc < 2) 
	{
		printf("\nNeed More Arguments\n");
		return -1;
	}
 
	printf("\nOpening File...\n"); 

	hFile = CreateFile(argv[1], GENERIC_READ, FILE_SHARE_READ, 
		0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE) 
	{
		printf("Cannot Open the File\n");
		return -1;
	}

	FileSize = GetFileSize(hFile, NULL);

	BaseAddress = (BYTE *) malloc(FileSize);

	if (!ReadFile(hFile, BaseAddress, FileSize, &BR, NULL))
	{
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}
 
	ImageDosHeader = (IMAGE_DOS_HEADER *) BaseAddress;

	// controlliamo il Dos Header
	if (ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		printf("Invalid Dos Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}

	ImageNtHeaders = (IMAGE_NT_HEADERS *) 
		(ImageDosHeader->e_lfanew + (DWORD) ImageDosHeader);

	// controlliamo il PE Header
	if (ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		printf("Invalid PE Header\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	if (!ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress)
	{
		printf("This PE doesn't contain Relocations\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	} 

	// converte in offset fisico l'RVA
	Reloc_Offset = RvaToOffset(ImageNtHeaders, 
		ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress);

	Reloc_Size = ImageNtHeaders->OptionalHeader.DataDirectory
		[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size;

	//  valido?
	if (Reloc_Offset == NULL)
	{
		printf("This PE doesn't contain Relocations\n");
		free(BaseAddress);
		CloseHandle(hFile);
		return -1;
	}  

	printf("\nRelocations:");

	// continua il ciclo finch
	// la size attuale  minore 
	// della totale dimensione
	// della Reloc Table
	while (Size < Reloc_Size)
	{
		// ricavo l'attuale reloc base
		ImageRelocation = (IMAGE_BASE_RELOCATION *)(DWORD)
			(Reloc_Offset + Size + (DWORD) BaseAddress);

		// stampo le info relative alla reloc base
		printf("\n\nVirtual Addr: %X\n"
			"Size Of Block: %X\n"
			"Type Offset Members %d",
			ImageRelocation->VirtualAddress,
			ImageRelocation->SizeOfBlock,
			((ImageRelocation-> SizeOfBlock -
			IMAGE_SIZEOF_BASE_RELOCATION) / sizeof (WORD)));

		// somma al size attuale la grandezza
		// della reloc attuale
		Size += ImageRelocation->SizeOfBlock;
	}


	free(BaseAddress);
	CloseHandle(hFile);

	return 0;
}

// sappiamo a cosa serve

DWORD RvaToOffset(IMAGE_NT_HEADERS *NT, DWORD Rva)
{
	DWORD Offset = Rva, Limit;
	IMAGE_SECTION_HEADER *Img;
	WORD i;

	Img = IMAGE_FIRST_SECTION(NT);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (i = 0; i < NT->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Rva >= Img[i].VirtualAddress &&
		    Rva < (Img[i].VirtualAddress + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].VirtualAddress;
				Offset += Img[i].PointerToRawData;
			}
				
			return Offset;
		}
	}

	return NULL;
}